/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

define([
    'underscore',
    'jquery',
    'mage/translate',
    'jquery/ui',
    'Magento_Customer/js/customer-data',
    'ko'
], function (_, $, $t, ui, customerData,ko) {
    'use strict';

    $.widget('filoblu.addToWishlist', {
        options: {
            bindSubmit: true,
            productSku: null,
            messagesSelector: '[data-placeholder="messages"]',
            addToWishlistCheckedClass: 'wished',
            textAdd: $t('Add to Wish List'),
            textRemove: $t('Remove from Wish List')
        },

        /** @inheritdoc */
        _create: function () {
            var self = this;

            if (this.options.bindSubmit) {
                this._bindSubmit();
            }

            this.checkProductInWishlist = ko.computed(function() {

                var found = false;

                var wishlist = customerData.get('wishlist');

                if (wishlist) {

                    $(wishlist().items).each(function (key, value) {
                        if(self.options.productSku == value.product_sku){
                            self.element.addClass(self.options.addToWishlistCheckedClass).attr('data-product-id', value.product_id);
                            $('span', self.element).text(self.options.textRemove);
                            $(self.element).attr('title', self.options.textRemove);
                            $(self.element).attr('aria-label', self.options.textRemove);
                            found = true;
                        }
                    });
                }

                return found;

            }, this);
        },

        /**
         * @private
         */
        _bindSubmit: function () {
            var self = this;

            this.element.on('click', function (e) {
                e.preventDefault();

                if(!$(this).hasClass(self.options.addToWishlistCheckedClass)){
                    self.addItem($(this), $("form#form-"+$(this).attr('id')));
                }else{
                    self.removeItem($(this), $("form#form-"+$(this).attr('id')));
                }
            });
        },

        addItem: function (button, form) {
            var self = this;

            var params = jQuery.parseJSON(button.attr('data-params'));
            var data = params.data;

            $.ajax({
                url: params.action,
                data: form.serialize() + '&action=' + params.action + "&product=" + data.product + "&uenc=" + data.uenc,
                type: 'post',
                dataType: 'json',

                /** @inheritdoc */
                beforeSend: function () {
                    $('body').trigger('processStart');
                },

                /** @inheritdoc */
                error: function(err){
                    $('body').trigger('processStop');
                    console.log(err);
                },

                /** @inheritdoc */
                success: function (res) {
                    var eventData, parameters;

                    customerData.reload(['wishlist'], false);

                    if (res.messages) {
                        $(self.options.messagesSelector).html(res.messages);
                    }

                    $('span', self.element).text(self.options.textRemove);
                    $(self.element).attr('title', self.options.textRemove);
                    $(self.element).attr('aria-label', self.options.textRemove);

                    $('body').trigger('processStop');

                }
            });
        },

        removeItem: function (button, form) {
            var self = this;

            var params = jQuery.parseJSON(button.attr('data-params-remove'));
            var data = params.data;

            $.ajax({
                url: params.action,
                data: form.serialize() + '&action=' + params.action + "&product_id=" + button.attr('data-product-id') + "&uenc=" + data.uenc,
                type: 'post',
                dataType: 'json',

                /** @inheritdoc */
                beforeSend: function () {
                    $('body').trigger('processStart');
                },

                /** @inheritdoc */
                error: function(err){
                    $('body').trigger('processStop');
                    console.log(err);
                },

                /** @inheritdoc */
                success: function (res) {
                    var eventData, parameters;

                    customerData.reload(['wishlist'], false);

                    if (res.messages) {
                        $(self.options.messagesSelector).html(res.messages);
                    }

                    $('body').trigger('processStop');

                    $(button).removeClass(self.options.addToWishlistCheckedClass);

                    $('span', self.element).text(self.options.textAdd);
                    $(self.element).attr('title', self.options.textAdd);
                    $(self.element).attr('aria-label', self.options.textAdd);

                }
            });
        }

    });

    return $.filoblu.addToWishlist;
});
